/*	$LAAS: t3dconvert2euler.c,v 1.2 2005/08/31 21:56:52 sara Exp $ */

/* 
 * Copyright (c) 2000-2004 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "t3d/t3d.h"
#include "t3d_internal.h"


/* Local functions --------------------------------------------------- */

static void	convertVectorToEuler(T3D *t3d);
static void	convertMatrixToEuler(T3D *t3d);


/*
 * --- t3dConvertToEuler ------------------------------------------------
 *
 * Convert T3D to Euler representation
 */

void
t3dConvertToEuler(T3D *t3d)
{
   switch(t3d->type) {
      case T3D_MATRIX: convertMatrixToEuler(t3d);	break; 
      case T3D_VECTOR: convertVectorToEuler(t3d);	break; 
      case T3D_EULER:					break;

      default:
	 t3d->any.err = T3D_ENOTIMPLEMENTED; break;
   }
}


/*
 * --- convertVectorToEuler ---------------------------------------------
 *
 * Convert vector T3D to Euler representation. Use the simple approach
 * that uses matrix as an intermediate representation. Since the variance
 * matrix is the same for Euler and matrix, this is not that costly.
 */

static void
convertVectorToEuler(T3D *t3d)
{
   t3dConvertToMatrix(t3d);
   convertMatrixToEuler(t3d);
}


/*
 * --- convertMatrixToEuler ---------------------------------------------
 *
 * Convert matrix T3D to Euler representation.
 */

static void
convertMatrixToEuler(T3D *t3d)
{
   double euler[6], cov[21];
  
   /* compute Euler parameters */
   t3dEulerOfMatrix(t3d->matrix.matrix, euler);

   /* copy covariance matrix */
   memcpy(cov, t3d->matrix.v, sizeof(cov));
  
   /* update t3d */
   memcpy(t3d->euler.euler, euler, sizeof(euler));
   memcpy(t3d->euler.v, cov, sizeof(cov));  
  
   t3d->type = T3D_EULER;
}
