/*	$Id: c-bsd2.c,v 1.1.1.1 2005/09/25 22:25:52 tho Exp $ */

/* 
 * Copyright (c) 2007 Anthony Mallet                  --  Thu Feb  1 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdlib.h>

#include "t3d/v3d.h"


/* --- v3dNew -------------------------------------------------------- */

/** \brief Malloc a new initialized structure for a V3D.
 *
 * Calls v3dInit() with an malloc'ed structure. Storage must be released
 * with v3dDelete().
 *
 * \return A pointer to the initialized V3D.
 */

V3D *
v3dNew()
{
   V3D *new;
   
   new = malloc(sizeof(V3D));
   if (new == NULL) return NULL;

   return v3dInit(new);
}


/* --- v3dInit ------------------------------------------------------- */

/** \brief Initialize a V3D.
 *
 * \param[in] v3d	Pointer to an allocated V3D.
 *
 * \return A pointer to the initialized V3D (equal to `v3d' parameter).
 */

V3D *
v3dInit(V3D *v3d)
{
  v3d->v[0] = v3d->v[1] = v3d->v[2] = 0.;
  v3d->v[4] = 1.;
  return v3d;
}


/*
 * v3dDelete ------------------------------------------------------------
 *
 * Free an malloc'ed T3D.
 */

void
v3dDelete(V3D *v3d)
{
   free(v3d);
}


/* --- v3dApplyT3D --------------------------------------------------- */

/** \brief Apply T3D to a V3D
 *
 * \param[in] t3d	Transformation.
 * \param[in] v3d	Vector to be transformed.
 *
 * \return the transformed vector (equal to `v3d').
 */

V3D *
v3dApplyT3D(T3D *t3d, V3D *v3d)
{
  V3D tmp;
  double *c, *v;
  int i, j;

  if (!t3dConvertTo(T3D_MATRIX, t3d)) return NULL;

  c = t3d->matrix.matrix;
  for(i=0; i<4; i++) {
    tmp.v[i] = 0.;
    v = v3d->v;
    for(j=0; j<4; j++) tmp.v[i] += *c++ * *v++;
  }

  *v3d = tmp;
  return v3d;
}
