/* 
 * Copyright (c) 2007 Anthony Mallet                  --  Thu Feb  1 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * $Id: h-bsd2.h,v 1.1.1.1 2005/09/25 22:25:52 tho Exp $
 */
#ifndef HPP_T3D
#define HPP_T3D

/** \file
 * C++ wrapper for T3D
 */

#include <iostream>

#include "t3d/t3d.h"
#include "t3d/v3d.hpp"

class CT3D {
 public:
  CT3D(T3DType type = T3D_MATRIX, int flags = 0) {
    t3dInit(&t3d, type, flags);
  }

  CT3D(const T3D *c) {
    t3dCopyIn(&t3d, c);
  }

  CT3D(const T3DMatrix &c) {
    this->t3d.matrix = c;
  }

  CT3D inv() {
    CT3D r(*this);
    
    t3dInvert(&r.t3d);
    return r;
  }

  CT3D operator*(CT3D &b) {
    CT3D r(*this);
    
    t3dComp(&r.t3d, &b.t3d);
    return r;
  }

  CV3D operator*(CV3D &b) {
    CV3D r(b);
    
    v3dApplyT3D(&this->t3d, &r.v3d);
    return r;
  }

  const double *matrix() {
    if (!t3dConvertTo(T3D_MATRIX, &this->t3d)) return NULL;
    return this->t3d.matrix.matrix;
  }

  double &matrix(int i) {
    t3dConvertTo(T3D_MATRIX, &this->t3d);
    return this->t3d.matrix.matrix[i];
  }

  double &euler(T3DEulerIndex i) {
    t3dConvertTo(T3D_EULER, &this->t3d);
    return this->t3d.euler.euler[i];
  }

 protected:
  T3D t3d;
};

class CR3D {

 private:
  int implement_me;
};

#endif /* HPP_T3D */
